package es.ucm.fdi.grasia.faerie.model.base;

import es.ucm.fdi.grasia.faerie.core.AbstractEntity;
import es.ucm.fdi.grasia.faerie.core.Attribute;
import es.ucm.fdi.grasia.faerie.core.Entity;
import es.ucm.fdi.grasia.faerie.core.EntitySet;
import es.ucm.fdi.grasia.faerie.core.util.ContextURIs;

public class AttributeConditionSet<E extends Entity, A extends Attribute<? super E, ? super T>, T> extends AbstractEntity
		implements EntitySet<E> {

	private static final long serialVersionUID = -53743429897943685L;
	private Class<E> entityType;
	private Class<A> attribute;
	private T value;
	private AttributeCondition condition = null;

	protected void resetId() {
		if (entityType != null && attribute != null)
			this.setId(ContextURIs.localEntityURI("/", entityType.getName() + "-" + attribute.getName() + "-"
					+ (value != null ? value.getClass().getName() + "@" : "") + System.identityHashCode(value)));
	}

	public AttributeConditionSet(Class<E> entityType, Class<A> attribute) {
		this.entityType = entityType;
		this.attribute = attribute;
		this.value = null;
		resetId();
	}

	public AttributeConditionSet(Class<E> entityType, Class<A> attribute, T value) {
		this.entityType = entityType;
		this.attribute = attribute;
		this.value = value;
		resetId();
	}

	public Class<E> getEntityType() {
		return entityType;
	}

	public void setEntityType(Class<E> entityType) {
		this.entityType = entityType;
		resetId();
	}

	public Class<A> getAttribute() {
		return attribute;
	}

	public void setAttribute(Class<A> attribute) {
		this.attribute = attribute;
		resetId();
	}

	public T getValue() {
		return value;
	}

	public void setValue(T value) {
		this.value = value;
		resetId();
	}

	public void setAttributeCondition(AttributeCondition condition) {
		this.condition = condition;
	}

	public AttributeCondition getAttributeCondition() {
		return condition;
	}

	@Override
	public String toString() {
		return "[e in " + entityType.getSimpleName() + " | e." + attribute.getSimpleName() + " " + condition + "]";
	}
}
