package es.ucm.fdi.grasia.faerie.model.base;

import es.ucm.fdi.grasia.faerie.core.Attribute;
import es.ucm.fdi.grasia.faerie.core.ContextValue;

/**
 *
 * @author Chema
 */
public class RelationalCondition<T extends Comparable<? super T>> implements AttributeCondition {

	public static final int DISTINCT = 0;
	public static final int LESS_THAN = 4;
	public static final int LESS_OR_EQUALS = 6;
	public static final int EQUALS = 2;
	public static final int GREATER_OR_EQUALS = 3;
	public static final int GREATER_THAN = 1;
	public static final int TRUE = 7;
	public static final int FALSE = 5;
	private int relation;
	private T value;

	public RelationalCondition(int relation, T value) {
		if (relation < 0 || relation >= 8)
			throw new IllegalArgumentException("relation not valid");
		this.relation = relation;
		this.value = value;
	}

	@Override
	public boolean resolve(Attribute attributeInstance) {
		ContextValue<T> contextValue = attributeInstance.getValue();
		if (contextValue != null) {
			T otherValue = contextValue.getData();
			if (otherValue != null) {
				int result = otherValue.compareTo(value);

				switch (relation) {
				case DISTINCT:
					return result != 0;
				case LESS_THAN:
					return result < 0;
				case LESS_OR_EQUALS:
					return result <= 0;
				case EQUALS:
					return result == 0;
				case GREATER_OR_EQUALS:
					return result >= 0;
				case GREATER_THAN:
					return result > 0;
				case TRUE:
					return true;
				case FALSE:
					return false;
				default:
					return false;
				}
			} else
				return relation == DISTINCT && value != null || relation == EQUALS && value == null || relation == TRUE;
		} else
			return relation == DISTINCT && value != null || relation == EQUALS && value == null || relation == TRUE;
	}

	@Override
	public String toString() {
		String result;
		switch (relation) {
		case DISTINCT:
			result = "!= " + value;
			break;
		case LESS_THAN:
			result = "< " + value;
			break;
		case LESS_OR_EQUALS:
			result = "<= " + value;
			break;
		case EQUALS:
			result = "== " + value;
			break;
		case GREATER_OR_EQUALS:
			result = ">= " + value;
			break;
		case GREATER_THAN:
			result = "> " + value;
			break;
		case TRUE:
			result = "== any";
			break;
		case FALSE:
			result = "!= any";
			break;
		default:
			result = "?";
			break;
		}
		return result;
	}
}
